@@ -1,11 +1,18 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
+ *               2010 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation. See COPYING file for details.
+ * modify it under the terms of the GNU General Public License version 2 
+ * or later as published by the Free Software Foundation. 
+ * See COPYING file for details.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "libisofs.h"
 #include "filesrc.h"
 #include "node.h"
 #include "util.h"
@@ -13,16 +20,28 @@
 #include "messages.h"
 #include "image.h"
 #include "stream.h"
-
-#ifdef Libisofs_with_checksumS
 #include "md5.h"
-#endif /*  Libisofs_with_checksumS */
 
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
 
 
+#ifdef Xorriso_standalonE
+
+#ifdef Xorriso_with_libjtE
+#include "../libjte/libjte.h"
+#endif
+
+#else
+
+#ifdef Libisofs_with_libjtE
+#include <libjte/libjte.h>
+#endif
+
+#endif /* ! Xorriso_standalonE */
+
+
 #ifndef PATH_MAX
 #define PATH_MAX Libisofs_default_path_maX
 #endif
@@ -52,11 +71,8 @@ int iso_file_src_create(Ecma119Image *img, IsoFile *file, IsoFileSrc **src)
     unsigned int fs_id;
     dev_t dev_id;
     ino_t ino_id;
-
-#ifdef Libisofs_with_checksumS
     int cret, no_md5= 0;
     void *xipt = NULL;
-#endif
 
     if (img == NULL || file == NULL || src == NULL) {
         return ISO_NULL_POINTER;
@@ -102,9 +118,6 @@ int iso_file_src_create(Ecma119Image *img, IsoFile *file, IsoFileSrc **src)
     /* insert the filesrc in the tree */
     ret = iso_rbtree_insert(img->files, fsrc, (void**)src);
     if (ret <= 0) {
-
-#ifdef Libisofs_with_checksumS
-
         if (ret == 0 && (*src)->checksum_index > 0) {
             /* Duplicate file source was mapped to previously registered source
             */
@@ -112,17 +125,12 @@ int iso_file_src_create(Ecma119Image *img, IsoFile *file, IsoFileSrc **src)
             if (cret < 0)
                 ret = cret;
         }
-
-#endif /* Libisofs_with_checksumS */
-
         free(fsrc->sections);
         free(fsrc);
         return ret;
     }
     iso_stream_ref(fsrc->stream);
 
-#ifdef Libisofs_with_checksumS
-
     if ((img->md5_file_checksums & 1) &&
         file->from_old_session && img->appendable) {
         ret = iso_node_get_xinfo((IsoNode *) file, checksum_md5_xinfo_func,
@@ -137,19 +145,17 @@ int iso_file_src_create(Ecma119Image *img, IsoFile *file, IsoFileSrc **src)
 
     if ((img->md5_file_checksums & 1) && !no_md5) {
         img->checksum_idx_counter++;
-        if (img->checksum_idx_counter < 0x80000000) {
+        if (img->checksum_idx_counter < 0x7fffffff) {
             fsrc->checksum_index = img->checksum_idx_counter;
         } else {
             fsrc->checksum_index= 0;
-            img->checksum_idx_counter= 0x80000000; /* keep from rolling over */
+            img->checksum_idx_counter= 0x7fffffff; /* keep from rolling over */
         }
         cret = iso_file_set_isofscx(file, (*src)->checksum_index, 0);
         if (cret < 0)
             return cret;
     }
 
-#endif /* Libisofs_with_checksumS */
-
     return ISO_SUCCESS;
 }
 
@@ -260,7 +266,12 @@ int filesrc_writer_compute_data_blocks(IsoImageWriter *writer)
         /*
          * final section
          */
-        file->sections[extent].block = t->curblock + extent * (ISO_EXTENT_SIZE / BLOCK_SIZE);
+        if (section_size <= 0) {
+            file->sections[extent].block = t->empty_file_block;
+        } else {
+            file->sections[extent].block =
+                         t->curblock + extent * (ISO_EXTENT_SIZE / BLOCK_SIZE);
+        }
         file->sections[extent].size = (uint32_t)section_size;
 
         t->curblock += DIV_UP(iso_file_src_get_size(file), BLOCK_SIZE);
@@ -303,21 +314,16 @@ int filesrc_read(IsoFileSrc *file, char *buf, size_t count)
     return iso_stream_read_buffer(file->stream, buf, count, &got);
 }
 
-#ifdef Libisofs_with_checksumS
-
 /* @return 1=ok, md5 is valid,
            0= not ok, go on,
           <0 fatal error, abort 
 */  
-
 static
 int filesrc_make_md5(Ecma119Image *t, IsoFileSrc *file, char md5[16], int flag)
 {
     return iso_stream_make_md5(file->stream, md5, 0);
 }
 
-#endif /* Libisofs_with_checksumS */
-
 static
 int filesrc_writer_write_data(IsoImageWriter *writer)
 {
@@ -330,14 +336,13 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
     char buffer[BLOCK_SIZE];
     off_t file_size;
     uint32_t nblocks;
-
-#ifdef Libisofs_with_checksumS
     void *ctx= NULL;
     char md5[16], pre_md5[16];
     int pre_md5_valid = 0;
+#ifdef Libisofs_with_libjtE
+    int jte_begun = 0;
 #endif
 
-
     if (writer == NULL) {
         return ISO_ASSERT_FAILURE;
     }
@@ -352,17 +357,11 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
         was_error = 0;
         file_size = iso_file_src_get_size(file);
         nblocks = DIV_UP(file_size, BLOCK_SIZE);
- 
-#ifdef Libisofs_with_checksumS
-
         pre_md5_valid = 0; 
         if (file->checksum_index > 0 && (t->md5_file_checksums & 2)) {
             /* Obtain an MD5 of content by a first read pass */
             pre_md5_valid = filesrc_make_md5(t, file, pre_md5, 0);
         }
-
-#endif /* Libisofs_with_checksumS */
-
         res = filesrc_open(file);
         iso_stream_get_file_name(file->stream, name);
         if (res < 0) {
@@ -406,18 +405,29 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
         }
 #endif
 
- 
-#ifdef Libisofs_with_checksumS
- 
+#ifdef Libisofs_with_libjtE
+        if (t->libjte_handle != NULL) {
+            res = libjte_begin_data_file(t->libjte_handle, name,
+                                         BLOCK_SIZE, file_size);
+            if (res <= 0) {
+                res = iso_libjte_forward_msgs(t->libjte_handle, t->image->id,
+                                        ISO_LIBJTE_FILE_FAILED, 0);
+                if (res < 0) {
+                    filesrc_close(file);
+                    ret = ISO_LIBJTE_FILE_FAILED;
+                    goto ex;
+                }
+            }
+            jte_begun = 1;
+        }
+#endif /* Libisofs_with_libjtE */
+
         if (file->checksum_index > 0) {
             /* initialize file checksum */
             res = iso_md5_start(&ctx);
             if (res <= 0)
                 file->checksum_index = 0;
         }
-
-#endif /* Libisofs_with_checksumS */
- 
         /* write file contents to image */
         for (b = 0; b < nblocks; ++b) {
             int wres;
@@ -433,9 +443,6 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
                 ret = wres;
                 goto ex;
             }
- 
-#ifdef Libisofs_with_checksumS
-
             if (file->checksum_index > 0) {
                 /* Add to file checksum */
                 if (file_size - b * BLOCK_SIZE > BLOCK_SIZE)
@@ -446,9 +453,6 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
                 if (res <= 0)
                     file->checksum_index = 0;
             }
-
-#endif /* Libisofs_with_checksumS */
-    
         }
 
         filesrc_close(file);
@@ -483,9 +487,6 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
                     ret = res;
                     goto ex;
                 }
- 
-#ifdef Libisofs_with_checksumS
-
                 if (file->checksum_index > 0) {
                     /* Add to file checksum */
                     if (file_size - b * BLOCK_SIZE > BLOCK_SIZE)
@@ -496,15 +497,10 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
                     if (res <= 0)
                         file->checksum_index = 0;
                 }
-
-#endif /* Libisofs_with_checksumS */
-    
             }
         }
-
-#ifdef Libisofs_with_checksumS
-
-        if (file->checksum_index > 0) {
+        if (file->checksum_index > 0 &&
+            file->checksum_index <= t->checksum_idx_counter) {
             /* Obtain checksum and dispose checksum context */
             res = iso_md5_end(&ctx, md5);
             if (res <= 0)
@@ -528,17 +524,33 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
             memcpy(t->checksum_buffer + 16 * file->checksum_index, md5, 16);
         }
 
-#endif /* Libisofs_with_checksumS */
+#ifdef Libisofs_with_libjtE
+        if (t->libjte_handle != NULL) {
+            res = libjte_end_data_file(t->libjte_handle);
+            if (res <= 0) {
+                iso_libjte_forward_msgs(t->libjte_handle, t->image->id,
+                                        ISO_LIBJTE_FILE_FAILED, 0);
+                ret = ISO_LIBJTE_FILE_FAILED;
+                goto ex;
+            }
+            jte_begun = 0;
+        }
+#endif /* Libisofs_with_libjtE */
 
     }
 
     ret = ISO_SUCCESS;
 ex:;
-
-#ifdef Libisofs_with_checksumS
     if (ctx != NULL) /* avoid any memory leak */
         iso_md5_end(&ctx, md5);
-#endif
+
+#ifdef Libisofs_with_libjtE
+    if (jte_begun) {
+        libjte_end_data_file(t->libjte_handle);
+        iso_libjte_forward_msgs(t->libjte_handle, t->image->id,
+                                ISO_LIBJTE_END_FAILED, 0);
+    }
+#endif /* Libisofs_with_libjtE */
 
     return ret;
 }
@@ -555,7 +567,7 @@ int iso_file_src_writer_create(Ecma119Image *target)
 {
     IsoImageWriter *writer;
 
-    writer = malloc(sizeof(IsoImageWriter));
+    writer = calloc(1, sizeof(IsoImageWriter));
     if (writer == NULL) {
         return ISO_OUT_OF_MEM;
     }
